<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\State;
use App\Models\County;
use App\Models\Client;
use App\Models\Service;
use App\Models\Order;
use App\Models\User;
use Storage;
use PDF;
use Session;
use DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;




class OrderCreationController extends Controller
{
      /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */

    public function single_order(){
        $stateList = State::select('id','short_code')->get();
        $mappedServices = DB::table('user_service_mapping')
                            ->where('user_id', Auth::id())
                            ->where('is_active', 1)
                            ->pluck('service_id')
                            ->toArray();

        if(Session::get('user_type_id') == 1 || Session::get('user_type_id') == 2) {
            $clientList = Client::select('id','client_name')->where('is_active', 1)->get();
            $serviceList = Service::select('id','service_name')->where('is_active', 1)->get();
        } else {
            $clientList = Client::select('id','client_name')->where('is_active', 1)->get();
            $serviceList = Service::select('id','service_name')->whereIn('id', $mappedServices)->where('is_active', 1)->get();
        }

        return view('app.orders.ordercreate' ,compact('stateList','clientList','serviceList'));
    }

    public function InsertOrder(Request $request){

        if(isset($request->is_vendor)) {
            $request->validate([
                'business_name' => 'required|regex:/^[a-zA-Z\s]+$/',
            ]);
        } else {
            $request->validate([
                'first_name' => 'required|regex:/^[a-zA-Z\s]+$/',
                'middle_name' => 'nullable|regex:/^[a-zA-Z\s]+$/',
                'last_name' => 'required|regex:/^[a-zA-Z\s]+$/',
                'first_name2' => 'nullable|regex:/^[a-zA-Z\s]+$/',
                'middle_name2' => 'nullable|regex:/^[a-zA-Z\s]+$/',
                'last_name2' => 'nullable|regex:/^[a-zA-Z\s]+$/',
                'ssn' => 'nullable|regex:/^\d{3}-\d{2}-\d{4}$/',
            ]);
        }

        $request->validate([
            'client' => 'required',
            'service' => 'required',
            'address1' => 'required',
            'address2' => 'nullable',
            'city' => 'required',
            'state' => 'required',
            'county' => 'nullable',
            'zip_code' => 'required|regex:/^\d{5}$/',
            'parcel_no' => 'nullable',
            'phone' => 'nullable',
            'address' => 'required',
            'property_city' => 'required',
            'property_state' => 'required',
            'property_county' => 'nullable',
            'property_zip_code' => 'required|regex:/^\d{5}$/',
            'legal_des' => 'nullable',
        ]);

        $data = $request->all();

        $OrderInfo = [
            'client_id' => $data['client'] ?? "",
            'service_id' => $data['service'] ?? "",
        ];

        if ($request->hasFile('legal_document')) {
            $filecontent = file_get_contents($request->file('legal_document'));
            $filename = uniqid('legal_document_') . '.' . 'pdf';
            \Storage::put('Legal_Document_Inputs/' . $filename, $filecontent);
            $legal_document = $filename;
        }


        $FirstBorrowerInfo = [
            'first_name' => isset($request->is_vendor) ? "" : ($data['first_name'] ?? ""),
            'middle_name' => isset($request->is_vendor) ? "" : ($data['middle_name'] ?? ""),
            'last_name' => isset($request->is_vendor) ? "" : ($data['last_name'] ?? ""),
            'address1' => $data['address1'] ?? "",
            'address2' => $data['address2'] ?? "",
            'city' => $data['city'] ?? "",
            'state' => !empty($data['state']) ? State::where('id', $data['state'])->value('short_code') : "",
            'county' => !empty($data['county']) ? County::where('id', $data['county'])->value('county_name') : "",
            'zip_code' => $data['zip_code'] ?? "",
            'parcel_no' => $data['parcel_no'] ?? "",
            'ssn' => $data['ssn'] ?? "",
            'phone' => $data['phone'] ?? "",
            'business_name' => isset($request->is_vendor) ? $data['business_name'] : ""
        ];

        $SecondaryBorrowerInfo = [
            'first_name' => isset($request->is_vendor) ? "" : ($data['first_name2'] ?? ""),
            'middle_name' => isset($request->is_vendor) ? "" : ($data['middle_name2'] ?? ""),
            'last_name' => isset($request->is_vendor) ? "" : ($data['last_name2'] ?? ""),
        ];

        $PropertyData = [
            'address' => $data['address'] ?? "",
            'city' => $data['property_city'] ?? "",
            'state' => !empty($data['property_state']) ? State::where('id', $data['property_state'])->value('short_code') : "",
            'county' => !empty($data['property_county']) ? County::where('id', $data['property_county'])->value('county_name') : "",
            'zip_code' => $data['property_zip_code'] ?? "",
            'legal_document' => $legal_document ?? "",
            'legal_desc' => $data['legal_des'] ?? "",
        ];

        $BorrowerInfo = [
            'FirstBorrowerInfo' => $FirstBorrowerInfo,
            'SecondaryBorrowerInfo' => $SecondaryBorrowerInfo,
        ];

        $responseData = [
            'OrderInfo' => $OrderInfo,
            'BorrowerInfo' => $BorrowerInfo,
            'PropertyData' => $PropertyData,
        ];

        $clientId = $OrderInfo['client_id'];
        $serviceId = $OrderInfo['service_id'];
        $inputData = json_encode($responseData);
        $user = User::where('id', Auth::id())->first();
        $clientList = Client::select('id','client_name')->get();
        $serviceList = Service::select('id','service_name')->get();
        $userId = $user['id'];

        $order = Order::create([
            'input' => $inputData,
            'client_id' => $clientId,
            'service_id' => $serviceId,
            'status' => 1,
            'last_updated_by' => $userId,
            'created_at' => now(),
        ]);

        if (!empty($order)) {
            return response()->json(['data' => 'success', 'msg' => 'Order Created Successfully']);
        } else {
            // return response()->json(['data' => 'warning', 'msg' => 'Check for Invalid Inputs']);
            return response()->json(['data' => 'error', 'msg' => 'Order Creation endup with Error']);
        }
    }
}
