<?php

namespace App\Traits;

use DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;

trait PdfTrait
{
    public function pdfMerger($orderId, $pdfIds)
    {
        $orderModel = app(\App\Models\Order::class);
        $supportingDocsModel = app(\App\Models\SupportingDocs::class);

        $orderPdfPath = $orderModel->where('id', $orderId)->value('pdf_path');
        $filePath = storage_path("app/{$orderPdfPath}/{$orderId}");
        $pdfFiles = ["{$filePath}/FilledForm.pdf"];

        $attachmentDocs = $supportingDocsModel->whereIn('id', $pdfIds)->pluck('pdf_file')->toArray();
        $pdfFiles = array_merge($pdfFiles, array_map(function ($doc) use ($filePath) {
            return "{$filePath}/{$doc}";
        }, $attachmentDocs));

        $startFileCount = count(File::files($filePath));

        $gs = (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') ? resource_path('gs9.55.0/bin/gswin64c.exe') : '/usr/bin/gs';
        $outputFile = "{$filePath}/Merged_Form.pdf";
        $command = "{$gs} -sDEVICE=pdfwrite -dNOPAUSE -dBATCH -dSAFER -sOutputFile={$outputFile}";

        foreach ($pdfFiles as $file) {
            $command .= ' ' . escapeshellarg($file);
        }

        $descriptorspec = [
            0 => ['pipe', 'r'],
            1 => ['pipe', 'w'],
            2 => ['pipe', 'w'],
        ];

        $process = proc_open($command, $descriptorspec, $pipes);

        if (is_resource($process)) {
            fclose($pipes[0]);
            fclose($pipes[1]);
            fclose($pipes[2]);
            while ($status = proc_get_status($process)) {
                if (!$status['running']) {
                    $exitCode = $status['exitcode'];
                    break;
                }
                usleep(100000);
            }
            $endFileCount = count(File::files($filePath));

            if($exitCode === 0 && $startFileCount + 1 == $endFileCount) {
                $supportingDocsModel->insert(['order_id' => $orderId, 'pdf_file' => "Merged_Form.pdf", 'created_at' => now()]);
                $response = "Merged PDF Generated Successfully";
            } else {
                $response = "Merged PDF Not Generated";
            }

            proc_close($process);
        } else {
            $response = "Failed to execute the shell command";
        }

        if(substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($filePath, 0777);
        }

        return response()->json($response);
    }

    protected function changeFolderPermissions($path, $permissions)
    {
        chmod($path, $permissions);
        $items = scandir($path);

        foreach ($items as $item) {
            if ($item != '.' && $item != '..') {
                $itemPath = $path . '/' . $item;
                if (is_dir($itemPath)) {
                    $this->changeFolderPermissions($itemPath, $permissions);
                } else {
                    chmod($itemPath, $permissions);
                }
            }
        }
    }

    public function upload_supporting_doc($orderId, $files, $baseDirPath)
    {
        $orderModel = app(\App\Models\Order::class);
        $supportingDocsModel = app(\App\Models\SupportingDocs::class);

        $order = $orderModel->find($orderId);
        $supportingdocids = [];
        $filePath = storage_path("app/{$baseDirPath}/{$order->id}");

        for ($i = 0; $i < count($files); $i++) {
            $file = $files[$i];
            $fileName = $file->getClientOriginalName();

            if ($order->supportingDocs()->where('pdf_file', $fileName)->count() > 0) {
                return response()->json(['message' => 'File already exists with the same name.']);
            }

            $startFileCount = count(File::files($filePath));
            $file->storeAs($filePath, $fileName);
            $endFileCount = count(File::files($filePath));

            if($startFileCount + 1 == $endFileCount) {
                DB::table('supporting_docs')->insert(['order_id' => $orderId, 'pdf_file' => $fileName,'created_at' => now()]);
            }

            $supportingdocids[] = $supportingDoc->id;
        }

        $supportingdocs = $supportingDocsModel->with('orders')->find($supportingdocids);

        if (substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($filePath, 0777);
        }

        return response()->json($supportingdocs);
    }

    public function delete_supporting_doc($orderId, $baseDirPath, $fileId)
    {
        $supportingDocsModel = app(\App\Models\SupportingDocs::class);
        $supportingDoc = $supportingDocsModel->with('order')->find($fileId);

        if ($supportingDoc) {
            $filePath = "$baseDirPath/{$supportingDoc->order_id}/{$supportingDoc->pdf_file}";
            if (Storage::exists($filePath)) {
                Storage::delete($filePath);
            }
            $supportingDocsModel->delete();
            return response()->json(['message' => 'File deleted successfully.']);
        }
        $supportingdocs = $supportingDocsModel->with('order')->where('order_id', $orderId)->get();
        return response()->json($supportingdocs);
    }

    public function  open_supporting_doc(Request $request)
    {
        $supportingDocsModel = app(\App\Models\SupportingDocs::class);
        $supportingdocs = $supportingDocsModel->with('order')->find($fileId);

        $filePath = 'storage/'.$supportingdocs->pdf_path."/".$supportingdocs->pdf_file;

        return Response::file($filePath, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="'. $supportingdocs->pdf_file .'"'
        ]);
    }
}
